#!/usr/bin/lua

local utils = require("lime.utils")
local config = require("lime.config")
local hardware_detection = require("lime.hardware_detection")

local libuci = require("uci")
local fs = require("nixio.fs")

usbradio = {}

usbradio.sectionNamePrefix = hardware_detection.sectionNamePrefix.."usbradio_"

function usbradio.clean()
	local uci = libuci:cursor()

	local function test_and_clean_device(s)
		local sectionName = s[".name"]
		if utils.stringStarts(sectionName, usbradio.sectionNamePrefix) then
			if config.get_bool(sectionName, "autogenerated") then
				local phyIndex = sectionName:match("%d+$")
				local _, numberOfMatches = fs.glob("/sys/devices/"..s["path"].."/ieee80211/phy"..phyIndex)
				if numberOfMatches < 1 then
					local radioName = s["radio_name"]
					uci:delete("wireless", radioName)
					config.delete(sectionName)
				end
			end
		end
	end

	uci:foreach("wireless", "wifi-device", test_and_clean_device)
	uci:save("wireless")
end


function usbradio.detect_hardware()
	local stdOutput = io.popen("find /sys/devices | grep usb | grep ieee80211 | grep 'phy[0-9]*$'")
	for path in stdOutput:lines() do
		local endBasePath, phyEnd = string.find(path, "/ieee80211/phy")
		local phyPath = string.sub(path, 14, endBasePath-1)
		local phyIndex = string.sub(path, phyEnd+1)
		local radioName = "radio"..phyIndex
		local sectionName = usbradio.sectionNamePrefix..radioName

		if config.autogenerable(sectionName) then
			local uci = libuci:cursor()

			uci:delete("wireless", radioName)
			uci:set("wireless", radioName, "wifi-device")
			uci:set("wireless", radioName, "type", "mac80211")
			uci:set("wireless", radioName, "channel", "11") --TODO: working on all 802.11bgn devices; find a general way for working in different devices
			uci:set("wireless", radioName, "hwmode", "11g") --TODO: working on all 802.11gn devices; find a general way for working in different devices
			uci:set("wireless", radioName, "path", phyPath)
			uci:set("wireless", radioName, "htmode", "HT20")
			uci:set("wireless", radioName, "disabled", "0")

			uci:save("wireless")

			config.init_batch()
			config.set(sectionName, "wifi")
			config.set(sectionName, "autogenerated", "true")
			config.set(sectionName, "radio_name", radioName)

			for option_name, value in pairs(config.get_all("wifi")) do
				if (option_name:sub(1,1) ~= ".") then
					if ( type(value) ~= "table" ) then value = tostring(value) end
					config.set(sectionName, option_name, value)
				end
			end
			
			local modes = {}
			for _, mode in pairs(config.get("wifi", "modes")) do
				if mode ~= "adhoc" then table.insert(modes, mode) end
			end
			config.set(sectionName, "modes", modes)

			config.end_batch()
		end
	end
end


return usbradio
